import { IPFS } from 'ipfs-core-types';
import { CID } from 'multiformats/cid';
import { Store } from '.';
import { PubSub } from './pubsub';
export interface Peer {
    id: string;
    address: string;
}
export interface PeerAnnouncement {
    kind: 'announcement';
    peer: Peer;
}
export declare type StackPubSubMessage = PeerAnnouncement;
export declare class Stack {
    namespace: CID;
    ipfs: IPFS;
    pubsub: PubSub<StackPubSubMessage>;
    store: Store;
    private announceInterval?;
    announce: boolean;
    constructor(namespace: CID, ipfs: IPFS);
    /**
     * get this peer info
     */
    id(): Promise<Peer>;
    private get libp2p();
    start(): Promise<void>;
    stop(): Promise<void>;
    /**
     * Create stack
     *
     * @param namespace needed to scope different stacks data/events
     * @param ipfs IPFS instance / see `@dstack-js/ipfs` package
     * @returns Stack instance
     */
    static create(namespace: string, ipfs: IPFS): Promise<Stack>;
    /**
     * Get connected peers
     *
     * @returns connected peers list
     */
    peers(): Promise<Peer[]>;
    /**
     * Connect to peer
     *
     * By default IPFS will connect to some peers automatically, no need to use it without a reason
     *
     * @param address MultiAddr to connect
     */
    connect(address: string): Promise<void>;
    /**
     * Listen on new peer connected
     *
     * @param listener Will be called with peer info
     */
    onPeerConnect(listener: (peer: Peer) => void): void;
    /**
     * Listen on peer disconnected
     *
     * @param listener Will be called with peer info
     */
    onPeerDisconnected(listener: (peer: Peer) => void): void;
    /**
     * Ping peer
     *
     * @param peer destination
     * @param options timeout and cycles settings `{ timeout: 1000, count: 10 }`
     * @returns round-trip time in ms
     */
    ping(peer: Peer, { timeout, count }: {
        timeout?: number | undefined;
        count?: number | undefined;
    }): Promise<number>;
}
