"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = void 0;
const tslib_1 = require("tslib");
const shard_1 = require("./shard");
class Store {
    constructor(stack) {
        this.stack = stack;
        this.data = {};
        this.pubsub = stack.pubsub.create('$$store');
    }
    onReplicate(msg) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (msg.data.kind !== 'replicate')
                return;
            const local = this.data[msg.data.key];
            const date = new Date(msg.data.date);
            if ((local === null || local === void 0 ? void 0 : local.date) > date)
                return;
            this.data[msg.data.key] = {
                value: msg.data.value,
                date
            };
        });
    }
    watchShard(key, watch) {
        watch.on('update', (shard) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.data[key] = {
                value: shard.toString(),
                date: new Date()
            };
            yield this.replicate(key);
        }));
    }
    get(key) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!this.data[key])
                return null;
            const shard = yield shard_1.Shard.from(this.stack, this.data[key].value);
            this.watchShard(key, shard);
            return shard;
        });
    }
    set(key, shard) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            this.data[key] = {
                value: shard.toString(),
                date: new Date()
            };
            this.watchShard(key, shard);
            yield this.replicate(key);
        });
    }
    replicate(key) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.pubsub.publish('replicate', {
                kind: 'replicate',
                date: this.data[key].date.toISOString(),
                key,
                value: this.data[key].value
            });
        });
    }
    start() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.pubsub.subscribe('replicate', (msg) => this.onReplicate(msg));
            yield this.pubsub.subscribe('replicateRequest', (msg) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                if (msg.data.kind !== 'replicateRequest')
                    return;
                yield Promise.all(Object.keys(this.data).map(this.replicate));
            }));
            yield this.pubsub.publish('replicateRequest', { kind: 'replicateRequest' });
        });
    }
    stop() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            yield this.pubsub.stop();
        });
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map